# -*- coding: UTF-8 -*-
#

#
# This file is part of ROSA Server Setup
#
# RSS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# RSS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RSS; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

import os
import select
import threading
from subprocess import Popen, PIPE, STDOUT

class ProcessThread(threading.Thread):
    """ Base class for running tasks """

    def __init__(self, command, cwd, callback, shell, language):
        #threading.Thread.__init__(self, **kwds)
        self.process = None
        self.command = command
        self.cwd = cwd
        self.callback = callback
        self._code = 2000
        self._output = ""
        self.lock = threading.RLock()
        self.shell = shell
        self.language = language
        threading.Thread.__init__(self)


    def get_output(self):
        return self._output.decode("UTF-8")
    output = property(get_output)


    def get_code(self):
        return self._code
    code = property(get_code)


    def run(self):
        """ run command """
        my_env = os.environ.copy()
        my_env["LANGUAGE"] = self.language
        self.process = Popen(self.command, stdout=PIPE, stderr=STDOUT,
            bufsize=1, cwd=self.cwd, env=my_env, close_fds=True, shell=self.shell)
        self.catch_output()
        return 0


    def catch_output(self):
        """ get command context """
        while self.isAlive():
            # get the file descriptor of the process stdout pipe
            # for reading
            try:
                fd = select.select([self.process.stdout.fileno()],
                    [], [], 5)[0][0]
            # raise an exception when the process doesn't make output
            # for long time
            except IndexError:
                fd = None
                pass

            self.process.poll()
            if self.process.returncode == None:
                # get bytes one by one
                if fd:
                    self.lock.acquire()
                    self._output += os.read(fd, 1)
                    self.lock.release()
            else:
                # get last bytes from output
                if fd:
                    self.lock.acquire()
                    self._output += os.read(fd, 4096)
                    self.lock.release()
                self._code = self.process.returncode
                if self.callback:
                    self.callback(self._code, self._output)
                break
