# -*- coding: UTF-8 -*-
#

#
# This file is part of ROSA Server Setup
#
# RSS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# RSS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RSS; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

import re
from IPy import IP

_ = lambda str: str


class Validation:
    """
    Data validation class

    """

    def fqdn(self, string):
        """ validate hostname """
        if not re.match('^[a-z0-9-\.]+\.[a-z]{2,}$', string):
            return [_("Incorrect %s"), "FQDN"]
        else:
            return None

    def network(self, networks):
        for ip, mask in networks:
            if not re.match('^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$', ip):
                return [_("Incorrect %s address"), "IP"]
            if not mask == "false":
        	print mask
                if not (re.match('^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$', mask) or re.match('^(([1-2][0-9])|(3[0-2])|([4-9]?))$', mask)):
                    return [_("Incorrect netmask address")]
        return None

    def ip(self, ips):
        for ip in ips:
            if not re.match('^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$', ip):
                return [_("Incorrect %s address"), "IP"]
        return None

    def is_smb_path(self, path):
        spath = path.strip()

        if not spath:
            return [_('The empty string')]

        # Windows

        # The trailing space is illegal

        if path != spath:
            return [_("The leading and the trailing spaces are illegal")]

        # The trailing dot is illegal

        if path[len(path) - 1] == '.':
            return [_("The trailing dot is illegal")]

        # Linux

        if path[0] != '/':
            return [_('The directory has to contain the staring slash')]

        sys_pattern = "^/$$|^/bin/{0,1}$|^/dev/{0,1}$|^/etc/{0,1}$|" + \
                      "^/home/{0,1}$|^/lib.*/{0,1}$|^/proc/{0,1}$|" + \
                      "^sys/{0,1}$|^sbin/{0,1}$|^usr/{0,1}$|^/var/{0,1}$"

        if re.match(sys_pattern, path):
            return [_('The system directory')]

        sys_virt_pattern = "^/dev/.*$|^/proc/.*$|^sys/.*$"
        if re.match(sys_virt_pattern, path):
            return [_('The system virtual directory')]

        # Windows

        # The illegal characters in the file name. > < | ? * / \ : "

        ic = re.findall('(<|>|\||\?|\*|\\\|:|")', path)
        if ic:
            return [_('The illegal characters are < > | ? * \ : "')]

        reserved_pattern = "^con$|^prn$|^aux$|^nul$|^com\d+$|^lpt\d+$"

        if re.match(reserved_pattern, path.split('/')[1], re.IGNORECASE):
            return [_("The reserved filenames are CON, PRN, AUX, NUL, COM1..COM9.., LPT1..LPT9..")]

        return None
